import logging
import re
import glob
import sys
import os
import subprocess
from typing import Dict

import json

from casa_envoy.auditor import AuditFiles, AuditDirectories

from casa_envoy.interfaces import LauncherIF
from casa_envoy.schema import AbstractTextFile


def get_abs_file(filename: str) -> AbstractTextFile:
    with open(filename) as file:
        if ".json" in filename:
            content = json.loads(file.read())
        elif ".xml" in filename:
            content = file.read()

    return AbstractTextFile(filename, content)


class CasaLauncher:
    def __init__(self, parameters: dict):
        self.logger = logging.getLogger("casa_envoy")
        self.parameters = parameters

    def setup_environment(self):
        os.environ["SCIPIPE_ROOTDIR"] = self.parameters["rootDirectory"]
        os.environ["CASA_HOME"] = self.parameters["homeForReprocessing"]
        os.environ["LANG"] = "en_US.UTF-8"
        os.environ["PPR_FILENAME"] = self.parameters["ppr"]

        self.check_processing_env()

    def check_processing_env(self):
        self.logger.info("Checking processing environment:")

        env_list = ["SCIPIPE_ROOTDIR", "CASA_HOME", "PPR_FILENAME", "LANG"]
        result_list = []

        for var in env_list:
            env = os.environ.get(var)
            result_list.append(env)
            self.logger.info(f"{var}: {env}")

        if "None" in result_list:
            self.logger.info("Environment setup Failed!")
            sys.exit(1)
        else:
            self.logger.info("Environment ready for processing")

    def run(self):
        self.setup_environment()

        if self.parameters.get("useCasa"):
            self.logger.info("RUNNING CASA!")
            os.chdir("./working")
            result = subprocess.Popen(
                "PYTHONPATH='' xvfb-run -e ${PWD}/xvfb-run.err.txt -d -s \"-screen 0 800x600x16\" "
                "${CASA_HOME}/bin/casa --pipeline --nogui --nologger -c "
                "${CASA_HOME}/pipeline/pipeline/runvlapipeline.py ${PPR_FILENAME} || true",
                shell=True,
                executable="/bin/bash",
                stdout=sys.stdout,
                stderr=sys.stderr,
            )
            return result.communicate()
        else:
            self.logger.info("RUNNING VELA!")
            run_type = self.parameters.get("product_type")
            metadata = self.parameters.get("metadata")
            ppr = self.parameters.get("ppr")
            subprocess.run(["./vela", run_type, metadata, ppr])

    def check_logs(self):
        self.logger.info("CHECKING CASA LOGS!")
        # make sure we are in the correct directory to find log file
        if not os.getcwd().endswith("/working"):
            os.chdir(self.parameters.get("parent_path") + "/working")

        casa_logs = glob.glob("casa-*.log")

        for file in casa_logs:
            if re.match("^.*SEVERE\sflagmanager.*$", open(file).read()):
                self.logger.error("CASA ERROR!")
            else:
                self.logger.info("CASA Success!")


class CalibrationLauncher(LauncherIF):
    def __init__(self, parameters: dict):
        self.logger = logging.getLogger("casa_envoy")
        self.parameters = parameters
        self.ppr = get_abs_file(parameters.get("ppr"))
        self.metadata = get_abs_file(parameters.get("metadata"))

    def launch_casa(self):
        if self.check_calibratable():
            self.run_audit(self.parameters)
            CasaLauncher(self.parameters).run()
        else:
            self.logger.error("ERROR: Provided SPL is not type execution block!")
            sys.exit(1)

    def check_calibratable(self) -> bool:
        spl = self.metadata.content["productLocator"]
        if "execblock" in spl:
            return True
        else:
            self.logger.info("SPL ERROR: This product locator is not calibratable!")
            return False

    def run_audit(self, parameters: Dict[str, str]):
        dir_audit = AuditDirectories(self.ppr, parameters).audit()
        if dir_audit:
            self.logger.info("Directory audit successful!")
        else:
            self.logger.error("FAILURE: directory structure audit was unsuccessful!")
            sys.exit(1)

        audit = AuditFiles([self.ppr, self.metadata], parameters).audit()
        if audit:
            self.logger.info("File audit successful!")
        else:
            self.logger.error("FAILURE: file audit was unsuccessful!")
            sys.exit(1)


class ImagingLauncher(LauncherIF):
    def __init__(self, parameters: dict):
        self.logger = logging.getLogger("casa_envoy")
        self.parameters = parameters
        self.ppr = get_abs_file(parameters.get("ppr"))
        self.metadata = get_abs_file(parameters.get("metadata"))

    def launch_casa(self):
        if self.check_imageable():
            self.run_audit(self.parameters)
            CasaLauncher(self.parameters).run()
        else:
            self.logger.error("ERROR: CMS information missing or incorrect!")
            sys.exit(1)

    def check_imageable(self) -> bool:
        content = self.metadata.content
        cms_name = content["cmsName"]
        cms_path = content["calibrationSourceDirectory"]
        if cms_name is not None and cms_path is not None and cms_name[-3:] == ".ms":
            return True
        else:
            self.logger.info("CMS ERROR: Imaging requires a valid CMS name and location!")
            return False

    def run_audit(self, parameters: Dict[str, str]):
        dir_audit = AuditDirectories(self.ppr, parameters).audit()
        if dir_audit:
            self.logger.info("Directory audit successful!")
        else:
            self.logger.info("FAILURE: directory structure audit was unsuccessful!")
            sys.exit(1)

        audit = AuditFiles([self.ppr, self.metadata], parameters).audit()
        if audit:
            self.logger.info("File audit successful!")
        else:
            self.logger.info("FAILURE: file audit was unsuccessful!")
            sys.exit(1)
