import json
import pathlib
from typing import Dict

from .context import DeliveryContext
from .finder import HeuristicProductFinder, ProductFinder


class Delivery:
    def has_piperesults(self, path: pathlib.Path) -> bool:
        raise NotImplementedError

    def create_product_finder(self, source: pathlib.Path) -> ProductFinder:
        """
        Based on the contents of the source/ folder, make the right flavor
        of product finder

        :param source: directory to examine
        :return: a product finder
        """
        # if there is a piperesults file, use that finder
        if self.has_piperesults(source):
            # return PiperesultsProductFinder(source)
            raise NotImplementedError
        else:
            return HeuristicProductFinder(source)

    def deliver(self, context: DeliveryContext) -> Dict:
        """
        Primary entrypoint to delivery process

        :param context:  Settings to configure delivery
        :return:  some sort of user-friendly representation of where everything wound up
        """
        # find the products
        finder = context.create_finder()

        # loop over the products and deliver them
        with context.create_destination() as destination:
            for product in finder.find_products():
                print(f"Delivering {product} to {destination}")
                product.deliver_to(destination)

        # the last thing we should do is return the URL to the delivered stuff.
        # for a local delivery, this will be a file URL; otherwise it will be an http URL.
        return destination.results()


def main(args=None):
    """CLI entry point"""
    # parse the arguments
    context = DeliveryContext.parse_commandline(args)

    # perform the delivery
    delivery = Delivery().deliver(context)

    # write the results to a file
    with open("delivery.json", "w") as result_file:
        # indent=2 causes the file to be pretty-printed for humans
        json.dump(delivery, result_file, indent=2)


if __name__ == "__main__":
    main()
