"""Build an ingestion manifest file"""
import json
import logging
import shutil
import sys
from pathlib import Path
from typing import Tuple, List

# pylint: disable=E0401, E0402
import pendulum
from pendulum import DateTime

from .ingestion_manifest import IngestionManifest

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))

MANIFEST_NAME_BASE = "ingestion_manifest_"
MANIFEST_NAME_EXT = ".json"
ARTIFACT_NAME = "ingestion_artifacts_"
ARTIFACT_EXT = ".tar"


class IngestionManifestWriter:
    """Uses supplied parameters to build ingestion manifest files
    for the various types of ingestion"""

    def __init__(
        self,
        manifest: IngestionManifest,
        working_dir: Path,
    ):
        self.manifest = manifest
        self.working_dir = working_dir
        self.parameters = manifest.parameters

        self.manifest_filename, self.artifact_filename = self.build_filenames()

    def build(self) -> Path:
        """
        Write the ingestion manifest indicated by the parameters.

        :return: the ingestion manifest constructed from the parameters
        """

        # Pull out the manifest content and stringify it
        manifest_content = json.dumps(self.manifest.content())

        # Write the manifest to the staging area, where other files may be.
        ingestion_location = self.parameters.ingestion_path
        staging_manifest = Path(ingestion_location) / self.manifest_filename
        with open(staging_manifest, "w") as out:
            out.write(manifest_content)

        # # Open up permissions so we can delete the manifest file later.
        # rw_mode = 0o666
        # staging_manifest.chmod(rw_mode)

        # Get all the files we'll need....
        ingestion_files = self.find_ingestion_files()
        # there should be at least one file, the manifest
        if len(ingestion_files) < 1:
            raise ValueError(f"no ingestion files nor manifest found at {ingestion_location}")

        # ....and put them in both places, with rw permissions
        for filename in ingestion_files:
            staging_dir_copy = ingestion_location / filename
            working_dir_copy = self.working_dir / filename
            shutil.copy(str(staging_manifest), str(working_dir_copy))

            # Rename the manifest to the shared name decided on for ingestion invocation.
            # Path() cast "shouldn't" be necessary, but if we don't do it,
            # file is a LocalPath and we can't create the symlink
            generalized = Path(self.working_dir / "ingestion-manifest.json")
            generalized.symlink_to(working_dir_copy, target_is_directory=False)

        # Now that all the loose files are copied, create the ingestion artifacts tar
        self.write_ingestion_artifact_tar(ingestion_files)

        # again: return a Path explicitly, for a LocalPath won't work
        return Path(staging_manifest)

    @staticmethod
    def format_timestamp(start_time: DateTime) -> str:
        """
        Format the current time as follows:
        input format:
            2021-07-01T13:49:17.237119+00:00
        desired output format as yyyy_MM_dd_'T'HH_mm_ss.SSS:
            2021_07_01'T'13_49_17.237

        :param start_time: current pendulum timestamp
        :return:
        """

        time_str = str(start_time)
        # change hyphens and colons to underscores
        timestamp = time_str.replace("-", "_", len(time_str))
        timestamp = timestamp.replace(":", "_", len(timestamp))
        # chop off the last bit
        timestamp = timestamp.split("+")[0]
        # now timestamp ends with ss.###....; round to 3 places
        ts_parts = timestamp.split("_")
        seconds = float(ts_parts[len(ts_parts) - 1])
        rounded = round(seconds, 3)
        timestamp = timestamp.replace(str(seconds), str(rounded))
        # finally, the T in single quotes
        timestamp = timestamp.replace("T", "'T'")

        return timestamp

    def build_filenames(self) -> Tuple[str, str]:
        """
        Build manifest and artifacts filenames.

        :return: the filenames
        """
        current_time = pendulum.now()
        timestamp = self.format_timestamp(current_time)
        manifest_filename = f"{MANIFEST_NAME_BASE}{timestamp}{MANIFEST_NAME_EXT}"
        artifact_filename = f"{ARTIFACT_NAME}{timestamp}{ARTIFACT_EXT}"
        return manifest_filename, artifact_filename

    def find_ingestion_files(self) -> List[Path]:
        """
        Gather the files required for ingestion

        :return: ingestion inputs
        """

        coll_files = aux_files = []

        if self.parameters.additional_metadata is not None:
            aux_string = self.parameters.additional_metadata
            aux_files = aux_string.split(",")

        if self.parameters.collection_metadata is not None:
            coll_str = self.parameters.collection_metadata
            coll_files = coll_str.split(",")

        aux_files += coll_files

        # be sure to add the manifest itself
        aux_files.append(self.manifest_filename)

        return aux_files

    def write_ingestion_artifact_tar(self, ingestion_files: List[Path]) -> Path:
        """
        Take the list of files and build a tar for inclusion into the archive.
        This happens in the staging area for ingestion.

        :param ingestion_files: all the files needed for ingestion
        :return: a .tar archive of the ingestion artifacts
        """
