""" Test for the various types of EVLA ingestion manifests """
import json
import logging
import re
import sys
from pathlib import Path

import pytest

from ingest_envoy.ingestion_manifest import IngestionManifest, Parameters
from ingest_envoy.ingestion_manifest_writer import IngestionManifestWriter
from ingest_envoy.ingestion_manifest_writer import (
    MANIFEST_NAME_BASE,
    MANIFEST_NAME_EXT,
)
from ingest_envoy.utilities import Telescope

# pylint: disable=E0401, W1203


logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


def test_minimal_manifest_is_as_expected(tmpdir):
    """
    Make sure a minimal, generic manifest has the properties we expect it to have.

    :return:
    """

    ingest_path = tmpdir / "ingestion"
    ingest_path.mkdir()

    # no reingest, ngas ingest, additional metadata nor collection metadata
    parameters = Parameters(Telescope.EVLA, ingest_path, None, None)

    manifest = IngestionManifest(parameters)

    fake_ancillary_product = {
        "type": "obslog",
        "filename": "my_fabulous_obs.log",
        "group_with": "uid://evla/execblock/ABCDEFGHIJKLMNOP",
    }
    manifest.output_group.ancillary_products.append(fake_ancillary_product)

    assert manifest.parameters == parameters
    assert len(manifest.output_group.science_products) == 0
    assert manifest.output_group.ancillary_products == [fake_ancillary_product]


def test_generates_correct_filename(tmpdir):
    """
    We expect the manifest to be named like "ingestion_manifest_2019_07_30_T13_03_00.936.json"

    :return:
    """
    ingest_path = tmpdir / "ingestion"
    ingest_path.mkdir()

    working_dir = tmpdir / "working"
    working_dir.mkdir()

    # no reingest, ngas ingest, additional metadata nor collection metadata
    parameters = Parameters(Telescope.EVLA, Path(ingest_path), None, None)

    manifest = IngestionManifest(parameters)
    manifest_file = IngestionManifestWriter(manifest, working_dir).build()
    filename = manifest_file.name
    assert filename.startswith(MANIFEST_NAME_BASE)
    assert filename.endswith(MANIFEST_NAME_EXT)

    filename_parts = filename.split("_")
    assert len(filename_parts) == 7

    # get just the timestamp
    timestamp = filename.replace(MANIFEST_NAME_BASE, "").replace(MANIFEST_NAME_EXT, "")

    # we should have gotten year, month, day, hours, minutes, seconds to 3 decimal places
    assert re.match(r"\d{4}_\d{2}_\d{2}'T'\d{2}_\d{2}_\d{2}\.\d{0,3}", timestamp)


def test_builds_evla_sdm_manifest(tmpdir):
    """
    Have we built a well-formed EVLA SDM ingestion manifest?

    :return:
    """

    ingest_path = tmpdir / "ingestion"
    ingest_path.mkdir()
    working_dir = tmpdir / "working"
    working_dir.mkdir()

    fake_science_product = json.dumps(
        {
            "type": "execution_block",
            "filename": "X_osro_000.59368.65423814815",
        }
    )
    fake_ancillary_product = json.dumps(
        {
            "type": "ingestion_artifacts",
            "filename": "ingestion_artifacts_2021_06_03_T15_52_35.031.tar",
        }
    )
    # no reingest, ngas ingest, additional metadata nor collection metadata
    parameters = Parameters(
        telescope=Telescope.EVLA,
        ingestion_path=str(ingest_path),
        additional_metadata=fake_science_product,
        collection_metadata=fake_ancillary_product,
    )

    manifest = IngestionManifest(parameters)

    assert len(manifest.input_group.science_products) == 1
    assert manifest.input_group.science_products[0] == json.loads(fake_science_product)
    assert len(manifest.output_group.ancillary_products) == 1
    assert manifest.output_group.ancillary_products == [json.loads(fake_ancillary_product)]


@pytest.mark.skip("TODO: test_writes_evla_sdm_manifest")
def test_writes_evla_sdm_manifest():
    """
    Have we written an EVLA BDF ingestion manifest correctly?

    :return:
    """
    # TODO:
    # manifest_file = IngestionManifestWriter(manifest, working_dir).build()
    raise NotImplementedError


@pytest.mark.skip("TODO: test_builds_evla_bdf_manifest")
def test_builds_evla_bdf_manifest():
    """
    Have we built a well-formed EVLA BDF ingestion manifest?

    :return:
    """
    raise NotImplementedError


@pytest.mark.skip("TODO: test_builds_evla_cal_manifest")
def test_builds_evla_cal_manifest():
    """
    Have we built a well-formed EVLA calibration ingestion manifest?

    :return:
    """
    raise NotImplementedError
